/***********************************
 *        Timothy Hannafin         *
 *        hannafit@wpi.edu         *
 * Worcester Polytechnic Institute *
 *               2006              *
 * *********************************/
using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.IO;

namespace RobotController
{
	/// <summary>
	/// The feedback server is responsible for sending the values of all 4 sensors, as well 
	/// as the current neural net detection status back to the client whenever it receives a request to do so
	/// </summary>
	public class FeedbackServer
	{
		private string address; 
		private int port;
		private Robot myRobot;
		private Controller form1;
		private NetworkStream feedbackStream = null;
		private bool exitServerFlag = true;

		public FeedbackServer(string addr, int prt, Robot arobot)
		{
			address = addr;
			port = prt;
			myRobot = arobot;
		}


		public void Start()
		{
			IPAddress addr = IPAddress.Parse(address);
			TcpListener listen;
			TcpClient feedbackClient;

			StreamReader reader;
			string line;

			byte[] bytes = new byte[16];

			while (exitServerFlag)
			{
				listen = new TcpListener(addr, port);
				listen.Start();
				feedbackClient = listen.AcceptTcpClient();
				feedbackStream = feedbackClient.GetStream();


				reader = new StreamReader(feedbackStream); 

				while(true)
				{
					
					if (listen != null)
					{

						try
						{
							line = reader.ReadLine(); //wait for a request from the client
						}
						catch
						{
							break;
						}
						if(line == "ext")
						{
							break;  //client has disconnected
						}
						else
						{	
							try  //send all the sensor data
							{
								bytes = Encoding.UTF8.GetBytes(myRobot.getLeftSensorValue().ToString() + "\n"); //send the left value
								feedbackStream.Write(bytes, 0, bytes.Length);

								bytes = Encoding.UTF8.GetBytes(myRobot.getFrontSensorValue().ToString() + "\n"); //send the front value
								feedbackStream.Write(bytes, 0, bytes.Length);

								bytes = Encoding.UTF8.GetBytes(myRobot.getRightSensorValue().ToString()+ "\n"); //send the right value
								feedbackStream.Write(bytes, 0, bytes.Length);

								bytes = Encoding.UTF8.GetBytes(myRobot.getBottomSensorValue().ToString()+ "\n"); //send the Bottom value
								feedbackStream.Write(bytes, 0, bytes.Length);

								//send an alert message if a human was detected or
								//there was an error in navigation

								if(myRobot.Status == StatusType.anomoly_detected)
								{
									bytes = Encoding.UTF8.GetBytes("det\n");
									myRobot.Status = StatusType.idle;
									feedbackStream.Write(bytes, 0, bytes.Length);
									continue;
								}
								if(myRobot.Status == StatusType.navigation_error)
								{
									myRobot.stop();
									bytes = Encoding.UTF8.GetBytes("nav\n");
									myRobot.Status = StatusType.idle;
									feedbackStream.Write(bytes, 0, bytes.Length);
									continue;
								}

								bytes = Encoding.UTF8.GetBytes("\n");
								feedbackStream.Write(bytes, 0, bytes.Length);


							}
							catch{}
						}

					}

				}

				reader.Close();
				feedbackClient.Close();
				listen.Stop();
			}
		}
	}
}
