/***********************************
 *        Timothy Hannafin         *
 *        hannafit@wpi.edu         *
 * Worcester Polytechnic Institute *
 *               2006              *
 * *********************************/
using System;
using System.Timers;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.IO;

namespace RobotController
{
	/// <summary>
	/// The image server is responsible for sending the latest buffered image from the webcam to the client whenever
	/// it receives a request to do so
	/// </summary>
	public class ImageServer
	{
		private string address; 
		private int port;
		private bool exitServerFlag = true;

		private WebcamInterface camInterface;

		public ImageServer(string addr, int prt, Robot arobot, Controller ctrl, WebcamInterface wi)
		{
			address = addr;
			port = prt;
			camInterface = wi;
		}

		/// <summary>
		/// Start the image server and wait for a new connection
		/// </summary>
		public void Start()
		{
			

			IPAddress addr = IPAddress.Parse(address);
			TcpListener listen;
			TcpClient client;
			NetworkStream clientStream;
			StreamReader reader;
			string line;

			byte[] image;// = new byte[fs.Length];
			byte[] bytes = new byte[16];

			while (exitServerFlag) //while the server is still running
			{
				listen = new TcpListener(addr, port);
				listen.Start();
				client = listen.AcceptTcpClient();
				clientStream = client.GetStream();


				reader = new StreamReader(clientStream);

				while(true)  //while a client is connected
				{
					
					if (listen != null)
					{
						line = reader.ReadLine();
						if(line == "ext") //client has left
						{
							break;
						}
						else
						{	
							try
							{
								image = camInterface.getWebImage();  //get the newest buffered image from the webcam
								bytes = Encoding.UTF8.GetBytes(image.Length.ToString() + "\n"); //send the image size

								clientStream.Write(bytes, 0, bytes.Length);  //send the size
								clientStream.Write(image, 0, image.Length);  //send the image
							}
							catch{}
						}

					}

				}

				//when client disconnects close everything, reopen and wait for new connection
				reader.Close();
				client.Close();
				listen.Stop();
			}

		}
	}
}
